﻿#region "Copyright (C) Lenny Granovsky. 2005-2013"
//This program is free software distribute under the terms of the GNU General Public License as published by
//the Free Software Foundation. Please read AssemblyInfo.cs file for more information.
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Web.Enhancements.Rest.Filters
{
    public class RequiresSslFilter : IWebRestFilter
    {
        /// <summary>
        /// Gets or sets the if REST service method requires SSL.
        /// </summary>
        public bool RequiresSsl { get; set; }

        public RequiresSslFilter()
        {
        }


        /// <summary>
        /// Method that is called prior the REST service handler created. This allows to cancel the call at earlier stages. 
        /// </summary>
        /// <remarks>The session and list parameters are not available at this time.</remarks>
        /// <param name="httpContext">Current HTTP content.</param>
        /// <param name="serviceContext">Current REST service context.</param>
        public void BeforeHandlerCreated(System.Web.HttpContext httpContext, IWebRestServiceContext serviceContext)
        {
            //verify if method requires SSL
            if (this.RequiresSsl && !httpContext.Request.IsSecureConnection)
                throw new WebRestException(ErrorMessages.ResourceManager.GetString("NonSecureNotAllowed"), serviceContext.MethodInfo.Name);
        }

        public void BeforeMethodCall(System.Web.HttpContext httpContext, IWebRestServiceContext serviceContext)
        {
        }

        public void AfterMethodCall(System.Web.HttpContext context, ref object returnValue)
        {
        }
    }
}
