﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Web.Enhancements.Rest.Templates
{
    /// <summary>
    /// Class that provides ability to return data through REST services that should be 
    /// processed as JSON-formatted data to respond back to the client. 
    /// This class uses DataContract to output the JSON data.
    /// </summary>
    internal class DataContractJsonTemplate : ITemplatedResult
    {
        /// <summary>
        /// Gets or sets result data that should be returned to the client.
        /// </summary>
        public object ResultData { get; set; }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public DataContractJsonTemplate()
        {
        }

        /// <summary>
        /// Contsractor that accpets result data that class should process into JSON response.
        /// </summary>
        /// <param name="resultData">Result data as an object</param>
        public DataContractJsonTemplate(object resultData)
        {
            this.ResultData = resultData;
        }

        /// <summary>
        /// Method converts ResultData object to JSON format and renders content into given HTTP response object.
        /// </summary>
        /// <param name="response">HTTP response object.</param>
        public void RenderResult(System.Web.HttpResponse response)
        {
            System.Runtime.Serialization.Json.DataContractJsonSerializer jsonSerializer = new System.Runtime.Serialization.Json.DataContractJsonSerializer(this.ResultData.GetType());
            jsonSerializer.WriteObject(response.OutputStream, this.ResultData);
        }
    }
}
